﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using CommonDll;

namespace MainApplication
{
    public partial class MainForm : Form
    {
        Assembly radControlsHost = null;
        Assembly controlsHost = null;
        DataTable dt = new DataTable();

        public MainForm()
        {
            InitializeComponent();

            try
            {
                System.IO.DirectoryInfo p = new System.IO.DirectoryInfo(@"..\bin");
                System.IO.FileInfo f = new System.IO.FileInfo(p.FullName + @"\RadControlsHostProvider.dll");
                radControlsHost = Assembly.LoadFile(f.FullName);             
            }
            catch
            {
                
            }

            try
            {
                System.IO.DirectoryInfo p = new System.IO.DirectoryInfo(@"..\bin");
                System.IO.FileInfo f = new System.IO.FileInfo(p.FullName + @"\ControlsHostProvider.dll");
                controlsHost = Assembly.LoadFile(f.FullName);
            }
            catch
            {

            }

            SetupDataSource();

            // In case both assemblies are loaded, we first check if RadControls are loaded
            if (radControlsHost != null)
            {
                ExploreTypes(radControlsHost);
                BindListControl("MyRadListControl");
                SetupWhiteButton("MyRadButton");  
                BindGrid("MyRadGridView");  
            }
            else if (controlsHost != null)
            {
                ExploreTypes(controlsHost);
                BindListControl("MyListBox");
                SetupWhiteButton("MyButton");          
                BindGrid("MyDataGridView");         
            }           
        }

        List<Type> ourTypes = new List<Type>();

        void ExploreTypes(Assembly assembly)
        {
            Type[] types = assembly.GetTypes();
           
            for (int i = 0; i < types.Length; i++)
            {
                if (typeof(CommonDll.IControlsTasks).IsAssignableFrom(types[i]))
                {
                    ourTypes.Add(types[i]);
                }
            }            
        }

        void SetupWhiteButton(string controlType)
        {
            foreach (Type type in ourTypes)
            {
                object obutton = Activator.CreateInstance(type);
                if (((IControlsTasks)obutton).ControlType == controlType)
                {
                    Control button = (Control)obutton;
                    button.Dock = DockStyle.Top;
                    button.Text = "Hi, I am a " + controlType + "!";
                    this.Controls.Add(button);

                    ((IControlsTasks)button).MyBackColor = Color.White;
                    break;
                }
            }
        }

        void BindGrid(string controlType)
        {
            foreach (Type type in ourTypes)
            {
                object ogrid = Activator.CreateInstance(type);
                if (((IControlsTasks)ogrid).ControlType == controlType)
                {
                    Control myGrid = (Control)ogrid;
                    myGrid.Dock = DockStyle.Fill;
                    myGrid.Text = "Hi, I am a " + controlType + "!";
                    this.Controls.Add(myGrid);

                    ((IControlsTasks)myGrid).MyBackColor = Color.Green;
                    ((IControlsTasks)myGrid).DataBind(DSource(dt), null, null);
                    break;
                }
            }
        }

        void BindListControl(string controlType)
        {
            foreach (Type type in ourTypes)
            {
                object olist = Activator.CreateInstance(type);
                if (((IControlsTasks)olist).ControlType == controlType)
                {
                    Control listControl = (Control)olist;
                    listControl.Dock = DockStyle.Bottom;
                    listControl.Text = "Hi, I am a " + controlType + "!";
                    this.Controls.Add(listControl);

                    ((IControlsTasks)listControl).MyBackColor = Color.Red;
                    ((IControlsTasks)listControl).DataBind(DSource(dt), "Name", "ID");
                    break;
                }
            }
        }

        BindingSource DSource(DataTable dTable)
        {
            BindingSource s = new BindingSource();
            s.DataSource = dTable;
            return s;
        }

        void SetupDataSource()
        {
            DataColumn dc = new DataColumn();
            dc.ColumnName = "ID";
            dt.Columns.Add(dc);

            dc = new DataColumn();
            dc.ColumnName = "Name";
            dt.Columns.Add(dc);

            for (int i = 0; i < 100; i++)
            {
                DataRow dr = dt.NewRow();
                dr[0] = i;
                dr[1] = "Name " + i.ToString();
                dt.Rows.Add(dr);
            }
        }
    }
}
